import 'package:flutter_firebase_basic_tools/widgets/custom_card.dart';
import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/widgets/user_info_form_container.dart';
import 'package:flutter_firebase_basic_tools/logic/user_info_form_functions.dart';

import 'package:flutter_firebase_basic_tools/logic/translate.dart';
import 'package:flutter/material.dart';
import 'package:flutter_firebase_basic_tools/flutter_firebase_basic_tools.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ForgotPassword extends StatefulWidget {
  const ForgotPassword({
    @required this.topbarBackgroundColor,
    @required this.userInfoFormContainerColor1,
    @required this.userInfoFormContainerColor2,
    @required this.userInfoFormContainerColor3,
    @required this.loginButtonColor,
    @required this.sendEmailButtonColor,
    @required this.sendEmailButtonGradientColor,
    @required this.languagePickerItemsTextColor,
    @required this.backgroundImagePath,
    @required this.languagePicker,
    @required this.openLogin,
    @required this.cardColor,
    @required this.cardElevation,
    @required this.customBackground,
    @required this.textList,
    @required this.logoImagePath,
    this.secretQuestionTextList,
  });

  final Color topbarBackgroundColor;
  final Color userInfoFormContainerColor1;
  final Color userInfoFormContainerColor2;
  final Color userInfoFormContainerColor3;
  final Color loginButtonColor;
  final Color sendEmailButtonColor;
  final LinearGradient sendEmailButtonGradientColor;
  final Color languagePickerItemsTextColor;
  final String backgroundImagePath;
  final bool languagePicker;
  final Function openLogin;
  final Color cardColor;
  final double cardElevation;
  final Widget customBackground;
  final List<String> textList;
  final String logoImagePath;
  final List<String> secretQuestionTextList;

  @override
  _ForgotPasswordState createState() => _ForgotPasswordState();
}

class _ForgotPasswordState extends State<ForgotPassword> {
  final GlobalKey<FormState> forgotPasswordFormKey = GlobalKey<FormState>();
  TextEditingController emailInputController = TextEditingController();
  TextEditingController answerInputController = TextEditingController();

  final GlobalKey<ScaffoldState> scaffoldKey = new GlobalKey<ScaffoldState>();

  String currentLanguage = "en";

  TranslationStream translationStream = TranslationStream();

  SharedPreferences prefs;

  String dropdownValue = "";

  updateTextList(int index, String newText) {
    widget.textList[index] = newText;
    setState(() {});
  }

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
    translationStream.init(widget.textList, updateTextList);
    translationStream.translate();
  }

  languagePickerCallback(String newCurrentLanguage) async {
    currentLanguage = newCurrentLanguage;
    translationStream.translate();
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
  }

  @override
  Widget build(BuildContext context) {
    bool isLandscape =
        (MediaQuery.of(context).orientation == Orientation.landscape);

    double appBarSize = 65;

    double sizedBoxSpace = 20;

    return Scaffold(
      key: scaffoldKey,
      extendBodyBehindAppBar: true,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: widget.topbarBackgroundColor,
          size: appBarSize,
          hasBackButton: true,
          actions: <Widget>[
            Container(
              margin: EdgeInsets.only(right: 20),
              width: isLandscape ? 200 : 100,
              child: widget.languagePicker
                  ? LanguagePicker(
                      currentLanguage: currentLanguage,
                      parentWidget: this,
                      function: languagePickerCallback,
                      languagePickerItemsTextColor:
                          widget.languagePickerItemsTextColor,
                    )
                  : Container(),
            ),
          ],
          customLeading: null,
          logoImage: "",
        ),
      ),
      body: UserInfoFormContainer(
        userInfoFormType: UserInfoFormType.forgotPassword,
        cardColor: widget.cardColor,
        cardElevation: widget.cardElevation,
        backgroundImagePath: widget.backgroundImagePath,
        linearGradient: LinearGradient(
          begin: FractionalOffset.bottomRight,
          end: FractionalOffset.topLeft,
          colors: [
            widget.userInfoFormContainerColor1,
            widget.userInfoFormContainerColor2,
            widget.userInfoFormContainerColor3,
          ],
          stops: [
            0.0,
            0.4,
            1.0,
          ],
        ),
        customBackground: widget.customBackground,
        children: [
          Container(
            height: 100,
            width: 300,
            decoration: BoxDecoration(
              image: DecorationImage(
                fit: BoxFit.contain,
                image: AssetImage(
                  widget.backgroundImagePath,
                ),
              ),
            ),
          ),
          SizedBox(
            height: sizedBoxSpace * 2,
          ),
          Text(
            widget.textList[0],
            textAlign: TextAlign.center,
            style: TextStyle(
              fontSize: 18,
              fontWeight: FontWeight.bold,
              color: widget.sendEmailButtonColor,
            ),
          ),
          SizedBox(
            height: sizedBoxSpace,
          ),
          TextFormField(
            decoration: InputDecoration(
              labelText: widget.textList[1],
              labelStyle: TextStyle(
                color: widget.sendEmailButtonColor,
              ),
            ),
            controller: emailInputController,
            validator: (String value) => FormFieldValidators(
              value: value,
              type: FormFieldValidatorsType.email,
            ).validate(),
          ),
          SizedBox(
            height: sizedBoxSpace,
          ),
          widget.secretQuestionTextList != null
              ? Container(
                  child: Column(
                    children: [
                      Container(
                        width: MediaQuery.of(context).size.width,
                        child: DropdownButton<String>(
                          isExpanded: true,
                          value: dropdownValue == ""
                              ? widget.secretQuestionTextList[0]
                              : dropdownValue,
                          icon: Icon(Icons.arrow_downward),
                          iconSize: 24,
                          elevation: 16,
                          style: TextStyle(color: widget.sendEmailButtonColor),
                          underline: Container(
                            height: 2,
                            color: widget.sendEmailButtonColor,
                          ),
                          onChanged: (String newValue) {
                            setState(() {
                              dropdownValue = newValue;
                            });
                          },
                          items: widget.secretQuestionTextList
                              .map<DropdownMenuItem<String>>((String value) {
                            return DropdownMenuItem<String>(
                              value: value,
                              child: Text(value),
                            );
                          }).toList(),
                        ),
                      ),
                      SizedBox(
                        height: sizedBoxSpace,
                      ),
                      TextFormField(
                        decoration: InputDecoration(
                          labelText: widget.textList[3],
                          labelStyle: TextStyle(
                            color: widget.sendEmailButtonColor,
                          ),
                        ),
                        controller: answerInputController,
                        validator: (String value) => FormFieldValidators(
                          value: value,
                          type: FormFieldValidatorsType.firstname,
                        ).validate(),
                      ),
                    ],
                  ),
                )
              : Container(),
          SizedBox(
            height: sizedBoxSpace * 2,
          ),
          Container(
            width: MediaQuery.of(context).size.width / 2,
            height: 60,
            child: CustomCard(
              borderRadius: MediaQuery.of(context).size.width,
              elevation: 7,
              onPressed: () {
                UserInfoFormFunctions().forgotPassword(
                  context: context,
                  forgotPasswordFormKey: forgotPasswordFormKey,
                  emailInputController: emailInputController,
                  openLogin: widget.openLogin,
                );
              },
              linearGradient: widget.sendEmailButtonGradientColor,
              color: widget.loginButtonColor,
              child: Center(
                child: Text(
                  widget.textList[2],
                  textAlign: TextAlign.center,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 16,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
