import 'package:flutter_firebase_basic_tools/widgets/background_image_with_gradient_color.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum UserInfoFormType {
  login,
  register,
  forgotPassword,
  accountInfo,
  custom,
}

class UserInfoFormContainer extends StatefulWidget {
  const UserInfoFormContainer({
    this.userInfoFormType,
    this.backgroundImagePath,
    this.linearGradient,
    this.children,
    this.cardColor,
    this.cardElevation,
    this.customBackground,
    this.formKey,
  });

  final UserInfoFormType userInfoFormType;
  final String backgroundImagePath;
  final LinearGradient linearGradient;
  final List<Widget> children;
  final Color cardColor;
  final double cardElevation;
  final Widget customBackground;
  final GlobalKey<FormState> formKey;

  @override
  _UserInfoFormContainerState createState() => _UserInfoFormContainerState();
}

class _UserInfoFormContainerState extends State<UserInfoFormContainer> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    bool isLandscape =
        (MediaQuery.of(context).orientation == Orientation.landscape);

    double height;
    double width;

    switch (widget.userInfoFormType) {
      case UserInfoFormType.login:
        {
          height = isLandscape ? 700 : MediaQuery.of(context).size.height;
          width = isLandscape ? 430 : MediaQuery.of(context).size.width;
        }
        break;

      case UserInfoFormType.register:
        {
          height = 600;
          width = isLandscape ? 650 : 330;
        }
        break;

      case UserInfoFormType.forgotPassword:
        {
          height = isLandscape ? 600 : MediaQuery.of(context).size.height;
          width = isLandscape ? 430 : MediaQuery.of(context).size.width;
        }
        break;

      case UserInfoFormType.accountInfo:
        {
          height = 600;
          width = isLandscape ? 650 : 330;
        }
        break;

      case UserInfoFormType.custom:
        {
          height = 450;
          width = isLandscape ? 430 : 330;
        }
        break;
    }

    if (widget.customBackground != null) {
      return Stack(
        children: [
          widget.customBackground,
          Center(
            child: SingleChildScrollView(
              child: Container(
                height: height,
                width: width,
                child: Card(
                  elevation: widget.cardElevation,
                  color: widget.cardColor,
                  child: Container(
                    padding: EdgeInsets.only(
                      left: 40,
                      right: isLandscape
                          ? 30
                          : widget.userInfoFormType ==
                                  UserInfoFormType.accountInfo
                              ? 10
                              : 40,
                    ),
                    child: Form(
                      key: widget.formKey,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: widget.children,
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ),
        ],
      );
    } else {
      return Container();
    }
  }
}
